/*
 * Copyright (c) 1999 - 2001, Artur Merke <amerke@ira.uka.de> 
 *
 * This file is part of FrameView2d.
 *
 * FrameView2d is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * FrameView2d is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FrameView2d; see the file COPYING.  If not, write to
 * the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 */


#ifndef _CONV_H_
#define _CONV_H_

#include <iostream.h>
#include "angle.h"
#include "point2d.h"

typedef double Value;

/**
  Die Klasse WinPlaneConverter konvertiert Koordinaten und Groessen der euklidischen
  Ebene in einen Fensterausschnitt (und umgekehrt).

  Ein Fenster ist def. durch

                   width 
            /---------------------\
            |                     |
    height  |			  |
            |                     |
            \---------------------/

  wobei die x-Koordinaten nach rechts wachsen (0 bis width)
  und die y-Koordinate von nach unten wachsen (0 bis height)
  Winkel sind in einem Fenster in Gradmass agegeben [0,...,360)

            | 90 Grad
	    |
   180      |          0 Grad
     -------O--------> 
	    |
  	    |
	    | 270 Grad

  ------------------------------------------------------------------------

  Ein Ausschnitt aus einer eukl. 2D Ebene ist definiert durch seine Linke
  untere Ecke (east,south) und sine rechte obere Ecke (west,north)
  Zugrunde liegt ein math. Koordniatensystem

            ^ Y axis
           /|\
	    |
            |
     -------O-------->  X axis
	    |
  	    |
	    |

  Winkel werden vor dem Umrechnen auf das Intervall [0,...,2*PI) normalisiert,
  
            
            | 0.5 Pi
	    |
   Pi       |          0
     -------O--------> 
	    |
  	    |
	    | 1.5 Pi

  Bei jeder Transformationen werden Laengenverhaeltnisse beruecksichtigt.
  Koordinaten, die ausserhalb der ausgezeichneten Bereiche liegen, werden
  so umgerechnet, als wenn die Bereiche keine Beschraenkungen haetten.
*/

class WinPlaneConverter {
friend ostream& operator<< (ostream& o, const WinPlaneConverter& c);
 public:
  WinPlaneConverter();
  WinPlaneConverter(int width, int height, 
		    const Point2d & center, Value size_x, Value size_y); 
  void init(int width, int height, 
	    const Point2d &center, Value size_x, Value size_y); 
  void init_win(int width, int height);
  void init_plane(const Point2d & center, Value size_x, Value size_y); 

  Value x_Win_to_x_Plane(int) const;
  Value y_Win_to_y_Plane(int) const;
  Value size_x_Win_to_size_x_Plane(int) const;
  Value size_y_Win_to_size_y_Plane(int) const;


  int x_Plane_to_x_Win(Value) const;
  int y_Plane_to_y_Win(Value) const;
  
  int size_x_Plane_to_size_x_Win(Value) const;
  int size_y_Plane_to_size_y_Win(Value) const;

  int angle_Plane_to_angle_Win(const Angle&) const;

  void get_Win(int & width, int & height) const;
  void get_Plane(Point2d & center, Value & size_x, Value & size_y) const;
 private:
  int Win_width;
  int Win_height;
  Value Plane_east_x;
  Value Plane_west_x;
  Value Plane_north_y;
  Value Plane_south_y;

};

ostream& operator<< (ostream& o, const WinPlaneConverter& c);
#endif
