/*
 * Copyright (c) 1999 - 2001, Artur Merke <amerke@ira.uka.de> 
 *
 * This file is part of FrameView2d.
 *
 * FrameView2d is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * FrameView2d is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FrameView2d; see the file COPYING.  If not, write to
 * the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 */
#ifndef _SMONITOR_DEV_H_
#define _SMONITOR_DEV_H_

#include "input_dev.h"
#include "rgbcolor.h"
#include "udpsocket.h"
#include "object2d.h"
#include <string.h>
#include "valueparser.h"

#define MAX_PLAYER 11

class SMonitorDevice: public InputDevice {
  UDPsocket server;
  //UDPsocket coach;

  static const int BUFFER_MAX_SIZE= 4096;
  static const int STRING_MAX_SIZE= 512;
  char buffer1[BUFFER_MAX_SIZE];
  char buffer2[BUFFER_MAX_SIZE];

  static const int frame_ball= 97;
  static const int frame_shadow= 98;
  static const int frame_canvas= 99;
  
  static const int BUTTON_START= 0;
  static const int BUTTON_RECONNECT= 1;
  static const int BUTTON_SCALE_LEVEL= 2;
  static const int BUTTON_INFO_LEVEL= 3;
  static const int BUTTON_QUIT= 4;
  static const int BUTTON_NUMBER= 5;
    
  struct Options {
    Options();
    int server_port;
    char server_host[STRING_MAX_SIZE];
    int coach_port;
    int protocol_version;

    bool list_player_types;
    bool just_edit;
    bool track_ball;
    double player_radius;
    int player_skin;

    double ball_radius;
    int ball_skin;

    double kick_radius;

    void toggle_track_ball() { track_ball= !track_ball; }

    RGBcolor c_team_l, c_goalie_l, c_font_l, c_invalid_l;
    RGBcolor c_team_r, c_goalie_r, c_font_r, c_invalid_r;
    RGBcolor c_ball, c_field, c_marigin, c_line, c_goal;

    int info_level;
    static const int info_level_min= 1;
    static const int info_level_def= 1;
    static const int info_level_max= 3;
    bool info_stamina() { return info_level > 1; }
    int scale_level;
    static const int scale_level_min= 0;
    static const int scale_level_def= 0;
    static const int scale_level_max= 5;
    static const double scale_level_inc= 0.2;
    static const double scale_level_beg= 1.0;
    double scale_factor() const { return scale_level_beg + scale_level * scale_level_inc; }
  };

  struct ServerState {
    //long last_packet_ms_time;
    struct Charfield {
      char	name[512] ;
      int len;
      Charfield() { name[0]= '\0'; len = 0;}
      void set( const char* str ) { strncpy(name,str,16); len= strlen(name); }
      void reset() { name[0]= '\0'; }
    };
    ServerState();
    Charfield left_teamname;
    Charfield right_teamname;
    Charfield match_information;
    bool reconnected;
    void reset();
  };

  struct Positions {
    struct Ball {
      Ball() { dragging= false; }
      Point2d pos;
      Point2d vel;

      bool dragging;
    };
    struct Player {
      Player() { alive= false; type= 0; dragging= false; };
      bool alive;
      int type;
      Point2d pos;
      Angle ang;

      bool dragging;
    };

    struct VisPlayer {
      VisPlayer() { str= 0; }
      VisualString2d * str;
    };

    Positions() {};
    Ball ball;
    Player player[MAX_PLAYER*2];
    VisPlayer vis_player[MAX_PLAYER*2];

    void set_player_alive(int i,bool flag=true) { player[i].alive= flag; }
    bool player_alive(int i) { return player[i].alive; }
    void set_player(int i,const Point2d & p,const Angle & a);
    void set_player_pos(int i,const Point2d & p) { player[i].pos= p; }
    void set_ball_pos(const Point2d & p) { ball.pos= p; }
    void set_ball(const Point2d & p, const Point2d & v) { ball.pos= p; ball.vel= v; }
    void print_inline(ostream & o);
    void print_formation(ostream & o, bool left_half, bool left_team);
  };

  struct CoachState {
    CoachState() { left_team= true; }
    bool left_team;
    void toggle_team() { left_team= !left_team; }
  };

  Options options;
  ServerState server_state;
  Positions server_pos;
  CoachState coach_state;

  const char * get_2D_object_par(const char * msg,int par_num,
				 double & p1, double & p2, double & p3, double & p4, 
				 char * color_str,int color_str_max_len );
  int server_msg_type(void * ptr);
  bool server_interpret_showinfo_t(BuilderBase * build, void *ptr);
  bool server_interpret_msginfo_t(BuilderBase * build, void *ptr);
  bool server_interpret_drawinfo_t(BuilderBase * build, void *ptr);

  bool server_interpret_showinfo_t2(BuilderBase * build, void *ptr); //ver. 2
  bool server_interpret_player_type_t(BuilderBase * build, void *ptr); //ver. 2


  /// the object ID of the ball
  static const int id_ball= -1;
  static const int id_invalid= -2;

  /// true if object ID means the ball or a valid player
  bool o_valid(int i) { return ( o_ball(i) || p_valid(i))?true:false; }
  /// true if object ID means the ball
  bool o_ball(int i) { return (i == id_ball)?true:false; }
  /// true if object ID means a player ball
  bool o_player(int i) { return p_valid(i); }
  /// true if the player ID is valid
  bool p_valid(int i) { return (i>= 0 && i < MAX_PLAYER*2)?true:false; }
  /// maps player ID to frame number of this player
  int p_frame(int i) { return i+1; }
  /// maps player ID to frame number of this player kick frame (with circle)
  int p_kick_frame(int i) { return i+30; }
  /// maps player ID to frame number of this player head frame
  int p_head_frame(int i) { return i+60; }
  /// maps player ID to number of this player in his team
  int p_number(int i) { return i<MAX_PLAYER?i+1:i+1-MAX_PLAYER; }
  /// maps player ID to team ID
  int p_team(int i) { return i<MAX_PLAYER?0:1; }
  /// return true if player ID belongs to the left team 
  bool p_left(int i) { return i<MAX_PLAYER?true:false; }
  /// return true if player ID belongs to the left team 
  bool p_right(int i) { return i<MAX_PLAYER?false:true; }
  /// maps player ID to character in '1' ... '9' , 'A', 'B' 
  char p_char(int i);

  ///sets player position, 
  void set_object_pos(BuilderBase * build, int p_num, const Point2d & pos);

  void set_all_objects_scale(BuilderBase * build, double scale);

  void send_object_pos(int p_num, const Point2d & pos);
  void send_dispinit();
  void send_dispstart();
  void send_dispball(const Point2d & pos);
  void send_dispfoul_left(const Point2d & pos);
  void send_dispfoul_right(const Point2d & pos);

  bool process_options(const ValueParser & vp);
 public:
  SMonitorDevice();

  bool process_options(const char * fname);
  bool process_options(int argc, char const* const* argv);
  void help_options(ostream & o);

  bool process_char_command(BuilderBase * build, char c, double x, double y);
  void help_char_command(ostream & o);

  bool process_mouse_button_event(BuilderBase * build, int button, int type, unsigned long time, const Point2d & pos);
  
  bool process_menu_button(BuilderBase * build, MenuBase * menu, int button, int mouse_button);
  
  bool init_menu(MenuBase * menu);
  bool init_frames(BuilderBase * build);
  bool init_connection();
  bool process_input(fd_set * , BuilderBase * build);

  const char * status_line() { return server_state.match_information.name; }
  int set_fds(fd_set * set);
  bool got_fds(fd_set * set);
};

#endif
